/*
 * Copyright (C) 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#ifndef GENERALNOTIFICATIONSETTINGS_H
#define GENERALNOTIFICATIONSETTINGS_H

// Qt
#include <QtCore/QObject>

#include <QGSettings/QGSettings>

class GeneralNotificationSettings : public QObject
{
    Q_OBJECT

    Q_PROPERTY(bool vibrateInSilentMode
               READ vibrateInSilentMode
               WRITE setVibrateInSilentMode
               NOTIFY vibrateInSilentModeChanged)

public:
    GeneralNotificationSettings(QObject* parent=0);
    ~GeneralNotificationSettings();

    bool vibrateInSilentMode();
    void setVibrateInSilentMode(bool vibrate);

Q_SIGNALS:
    void vibrateInSilentModeChanged();

protected Q_SLOTS:
    virtual void getSettings();

private Q_SLOTS:
    void onSettingsChanged(const QString& key);

private:
    QScopedPointer<QGSettings> m_settings;
    bool m_vibrateInSilentMode;
};

#endif // GENERALNOTIFICATIONSETTINGS_H
