/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -c AethercastManager -p aethercast_manager org.aethercast.xml org.aethercast.Manager
 *
 * qdbusxml2cpp is Copyright (C) 2015 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef AETHERCAST_MANAGER_H_1461342532
#define AETHERCAST_MANAGER_H_1461342532

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.aethercast.Manager
 */
class AethercastManager: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.aethercast.Manager"; }

public:
    AethercastManager(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~AethercastManager();

    Q_PROPERTY(QStringList Capabilities READ capabilities)
    inline QStringList capabilities() const
    { return qvariant_cast< QStringList >(property("Capabilities")); }

    Q_PROPERTY(bool Enabled READ enabled WRITE setEnabled)
    inline bool enabled() const
    { return qvariant_cast< bool >(property("Enabled")); }
    inline bool setEnabled(bool value)
    { return setProperty("Enabled", QVariant::fromValue(value)); }

    Q_PROPERTY(bool Scanning READ scanning)
    inline bool scanning() const
    { return qvariant_cast< bool >(property("Scanning")); }

    Q_PROPERTY(QString State READ state)
    inline QString state() const
    { return qvariant_cast< QString >(property("State")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> RegisterMediaManager(const QDBusObjectPath &path)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(path);
        return asyncCallWithArgumentList(QStringLiteral("RegisterMediaManager"), argumentList);
    }

    inline QDBusPendingReply<> Scan()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Scan"), argumentList);
    }

    inline QDBusPendingReply<> UnregisterMediaManager(const QDBusObjectPath &path)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(path);
        return asyncCallWithArgumentList(QStringLiteral("UnregisterMediaManager"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace aethercast {
    typedef ::AethercastManager Manager;
  }
}
#endif
