/*
 * This file is part of system-settings
 *
 * Copyright (C) 2014 Canonical Ltd.
 *
 * Contact: Jonas G. Drange <jonas.drange@canonical.com>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SESSIONSERVICE_H
#define SESSIONSERVICE_H

#include <QtDBus/QDBusInterface>

class SessionService : public QObject
{
    Q_OBJECT

public:
    explicit SessionService (QObject *parent = 0);
    void reboot();

private:
    QDBusConnection m_systemBusConnection;
    QDBusInterface m_loginManager;

};

#endif // SESSIONSERVICE_H
